@@ -209,6 +209,44 @@ describe('Custom keywords', function () {
       testMultipleRangeKeyword({ type: 'number', macro: macroRange }, 2);
     });
 
+    it('should support resolving $ref without id or $id', function () {
+      instances.forEach(function (_ajv) {
+        _ajv.addKeyword('macroRef', {
+          macro: function (schema, parentSchema, it) {
+            it.baseId .should.equal('#');
+            var ref = schema.$ref;
+            var validate = _ajv.getSchema(ref);
+            if (validate) return validate.schema;
+            throw new ajv.constructor.MissingRefError(it.baseId, ref);
+          },
+          metaSchema: {
+            "type": "object",
+            "required": [ "$ref" ],
+            "additionalProperties": false,
+            "properties": {
+              "$ref": {
+                "type": "string"
+              }
+            }
+          }
+        });
+        var schema = {
+          "macroRef": {
+            "$ref": "#/definitions/schema"
+          },
+          "definitions": {
+            "schema": {
+              "type": "string"
+            }
+          }
+        };
+        var validate;
+        (function compileMacroRef () { validate = _ajv.compile(schema); }).should.not.throw();
+        shouldBeValid(validate, 'foo');
+        shouldBeInvalid(validate, 1, 2);
+      });
+    });
+
     it('should recursively expand macro keywords', function() {
       instances.forEach(function (_ajv) {
         _ajv.addKeyword('deepProperties', { type: 'object', macro: macroDeepProperties });
@@ -377,6 +415,7 @@ describe('Custom keywords', function () {
     it('should correctly expand macros in macro expansions', function() {
       instances.forEach(function (_ajv) {
         _ajv.addKeyword('range', { type: 'number', macro: macroRange });
+        _ajv.addKeyword('exclusiveRange', { metaSchema: {type: 'boolean'} });
         _ajv.addKeyword('myContains', { type: 'array', macro: macroContains });
 
         var schema = {
@@ -773,6 +812,7 @@ describe('Custom keywords', function () {
   function testRangeKeyword(definition, customErrors, numErrors) {
     instances.forEach(function (_ajv) {
       _ajv.addKeyword('x-range', definition);
+      _ajv.addKeyword('exclusiveRange', {metaSchema: {type: 'boolean'}});
 
       var schema = { "x-range": [2, 4] };
       var validate = _ajv.compile(schema);
@@ -811,6 +851,7 @@ describe('Custom keywords', function () {
   function testMultipleRangeKeyword(definition, numErrors) {
     instances.forEach(function (_ajv) {
       _ajv.addKeyword('x-range', definition);
+      _ajv.addKeyword('exclusiveRange', {metaSchema: {type: 'boolean'}});
 
       var schema = {
         "properties": {
@@ -939,6 +980,13 @@ describe('Custom keywords', function () {
       });
     });
 
+    it('should return instance of itself', function() {
+      var res = ajv.addKeyword('any', {
+        validate: function() { return true; }
+      });
+      res.should.equal(ajv);
+    });
+
     it('should throw if unknown type is passed', function() {
       should.throw(function() {
         addKeyword('custom1', 'wrongtype');
@@ -1041,6 +1089,15 @@ describe('Custom keywords', function () {
       validate(1) .should.equal(false);
       validate(2) .should.equal(true);
     });
+
+    it('should return instance of itself', function() {
+      var res = ajv
+        .addKeyword('any', {
+          validate: function() { return true; }
+        })
+        .removeKeyword('any');
+      res.should.equal(ajv);
+    });
   });
 
 
@@ -1128,4 +1185,50 @@ describe('Custom keywords', function () {
       });
     });
   });
+
+
+  describe('"dependencies" in keyword definition', function() {
+    it("should require properties in the parent schema", function() {
+      ajv.addKeyword('allRequired', {
+        macro: function(schema, parentSchema) {
+          return schema ? {required: Object.keys(parentSchema.properties)} : true;
+        },
+        metaSchema: {type: 'boolean'},
+        dependencies: ['properties']
+      });
+
+      var invalidSchema = {
+        allRequired: true
+      };
+
+      should.throw(function () {
+        ajv.compile(invalidSchema);
+      });
+
+      var schema = {
+        properties: {
+          foo: true
+        },
+        allRequired: true
+      };
+
+      var v = ajv.compile(schema);
+      v({foo: 1}) .should.equal(true);
+      v({}) .should.equal(false);
+    });
+
+    it("'dependencies'should be array of valid strings", function() {
+      ajv.addKeyword('newKeyword1', {
+        metaSchema: {type: 'boolean'},
+        dependencies: ['dep1']
+      });
+
+      should.throw(function () {
+        ajv.addKeyword('newKeyword2', {
+          metaSchema: {type: 'boolean'},
+          dependencies: [1]
+        });
+      });
+    });
+  });
 });
